// Copyright (C) 2015 André Bargull. All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
es6id: 19.5.6.1
description: >
  EvalError is a constructor function.
---*/

assert.sameValue(typeof EvalError, 'function', 'typeof EvalError is "function"');
